.installed-overlay-box {
	font-size: 80%;
	background-color: @theme_selected_bg_color;
	border-radius: 4px;
	color: @theme_selected_fg_color;
}

.popular-installed-overlay-box {
	font-size: 80%;
	background-color: @theme_selected_bg_color;
	border-radius: 0;
	color: @theme_selected_fg_color;
	text-shadow: none;
}

.index-title-alignment-software {
	font-weight: bold;
	font-size: 14px;
}

.needs-attention {
	background-image: none;
	background-color: shade(@theme_selected_bg_color, 1.1);
	border-radius: 1px;
}

.screenshot-image, .screenshot-image-thumb {
	background-image: none;
	background-color: shade(@theme_bg_color, 0.9);
}
.screenshot-image { border-radius: 5px; }
.screenshot-image-thumb { border-radius: 3px; }

.app-tile-label {
	font-size: 10px;
}

.view.tile {
	padding: 0;
	background-image: none;
	background-color: mix(@theme_base_color,@theme_bg_color,0.3);
}

.view.tile:hover {
	background-color: @theme_base_color;
}

.view.tile:active {
	border-color: @theme_selected_bg_color;
	box-shadow: none;
	color: @theme_selected_bg_color;
}

.view.tile:backdrop {
	box-shadow: none;
	border-color: @unfocused_borders;
}

/* The rest of the featured-tile CSS is loaded at runtime in gs-feature-tile.c */
.button.featured-tile {
	all: unset;
	padding: 0;
	border-radius: 0;
	border-width: 1px;
	border-image: none;
}

.application-details-infobar.info {
	background-color: #d3d7cf;
	color: @theme_fg_color;
	border-color: darker(#d3d7cf);
	border-style: solid;
	border-width: 1px;
	text-shadow: none;
}

.application-details-infobar.warning {
	background-color: #fcaf3e;
	color: @theme_fg_color;
	border-color: darker(#fcaf3e);
	border-style: solid;
	border-width: 1px;
	text-shadow: none;
}

.application-details-title {
	font-weight: bold;
	font-size: 14px;
}

.application-details-summary {
}

.application-details-description {
}

.error-label {
	text-shadow: none;
}

.folder-label {
	font-size: 9px;
	background-color: #999999;
	border-radius: 4px;
	padding: 4px 8px;
	color: white;
}

.header-label {
	font-size: 11px;
	padding: 6px;
	background-image: none;
	background-color: #babdb6;
	border-color: #000000;
}

.image-list {
	background-color: transparent;
}

GtkBox.star {
	background-color: transparent;
	background-image: none;
}

.button.star {
	-GtkWidget-focus-padding: 0;
	background-color: transparent;
	background-image: none;
	border-image: none;
	border-radius: 0;
	border-width: 0px;
	padding: 0;
	box-shadow: none;
	outline-offset: -1px;
}

.star {
	color: #000000;
}

.star-user {
	color: @theme_selected_bg_color;
}

.counter-label {
	text-shadow: none;
	color: @theme_selected_fg_color;
	background-color: mix(@theme_selected_bg_color, @theme_selected_fg_color, 0.3);
	font-size: smaller;
	border-radius: 4px;
	padding: 0px 4px;
}

/* the following two selectors are to color the small gap before the list inside the scrolled window
   setting a background on the scrolled window affects the undershoot and the overshoot so explicitelly
   excluding with :not() */
.category-sidebar:not(.undershoot):not(.overshoot) { background-color: @theme_base_color; }

.category-sidebar:backdrop:not(.undershoot):not(.overshoot) { background-color: @theme_unfocused_base_color; }

/* Superfluous borders removal */
.category-sidebar {
	border-style: none;
}

.category-sidebar:dir(rtl) {
	border-left-style: solid;
}

.category-sidebar:dir(ltr) {
	border-right-style: solid;
}

.dimmer-label {
	opacity: 0.25;
	text-shadow: none;
}

.update-failed-details {
	font: Monospace;
	font-size: 90%;
	padding: 16px
}
